#!/usr/bin/csh -f

# Make random dot stereogram from PBM file.
# Requires rds program and Netpbm utilities.

# For "Drill Bits" column in July/August 2023 issue of
# ACM _Queue_ magazine:  https://queue.acm.org/DrillBits

if ("" == $1) then
    echo "usage: $0 name.pbm"
    echo "    where 'name' is bitmap of desired image"
    echo "    creates name.rnd.pbm, name.enc.pbm, and name.cat.pbm"
    exit
endif

set p = $1
set b = `basename $p .pbm`
set w = `gawk '2==NR{print $1}' $p`   # width of bitmap
set h = `gawk '2==NR{print $2}' $p`   # height of bitmap
set u = /dev/urandom   # source of random bytes
set r = ./rds          # random-dot stereogram generator program

# output files:
set k = $b.rnd.pbm    # random dot pattern
set e = $b.enc.pbm    # encodes image supplied by given bitmap
set c = $b.cat.pbm    # juxtaposes K and E files for easy viewing

if (-X valgrind) then
    set v = 'valgrind -q'
else
    echo 'valgrind not available'
    set v = ''
endif

echo 'create random dot pattern file K'
rm -f $k
$v $r k $w $h < $u > $k

echo 'create RDS:  P + K -> E'
rm -f $e
$v $r e $p $k < $u > $e

echo 'enlarge K and E, add border'
pamenlarge 1 < $k | pnmmargin -white 32 | pnmmargin -black 32 > foo ; mv -f foo $k
pamenlarge 1 < $e | pnmmargin -white 32 | pnmmargin -black 32 > foo ; mv -f foo $e

echo 'concat K and E for easy viewing'
rm -f $c
pamcat -lr $k $e > $c

