#!/usr/bin/csh -f

# C-Shell script to compile rds program.

# For "Drill Bits" column in July/August 2023 issue of
# ACM _Queue_ magazine:  https://queue.acm.org/DrillBits

# The gcc flag "-Wall" is misnamed; it does not enable all warnings.
# Not even "-Wall -Wextra" enable all warnings.  Below is one way to
# enable literally all warnings:  Run "gcc --help=warnings" to get a
# list of the warnings supported by your version of gcc and then
# manually re-format the list into an AWK-friendly format.  That's
# how I created my "warnflags" file.  See the "Schrodinger's Code"
# episode of "Drill Bits" for more on warning flags.
if (-e warnflags) then
    set wf = `gawk '/^[cx]/{print $2}' warnflags`
else
    echo 'using bare-minimum warning flags'
    set wf = '-Wall -Wextra'
endif

# CFLAGS analogue.  Manually disables a handful of warnings.
set cf = "-std=c11 $wf -Wno-c90-c99-compat -Wno-traditional-conversion -Wno-arith-conversion -Wno-suggest-attribute=pure"

# compile twice:  once for max warnings, once for debuggability
echo '' ; rm -f rds ; gcc $cf -O3 -o rds rds.c ; echo '' ; echo ''
echo '' ; rm -f rds ; gcc $cf -Og -o rds rds.c ; echo ''

