// Arithmetic mod 26.  Compile as C11.
// Accompanies "Drill Bits" column in July/August 2023 issue of
// ACM _Queue_ magazine:  https://queue.acm.org/DrillBits
// Author:  Terence Kelly <tpkelly@acm.org>

#include <assert.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "ck.h"

#define BUFLEN 1024

static const char A[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";  // alphabet
#define ALEN ((int)(sizeof A - 1))  // alphabet size excluding nul
#define AMOD (9 * ALEN)  // largest multiple of ALEN less than 256
static_assert(26 == ALEN && 256 >= AMOD, "bad parameters");

static int let2num(int c) {
  char *pos = strchr(A, c);  // slow
  CK(NULL != pos, "char '%c' not in alphabet '%s'", c, A);
  return (int)(pos - A);
}

static void check_string(const char *p) {
  size_t pos = strspn(p, A);
  CK(strlen(p) <  BUFLEN, "string '%s' too long",              p);
  CK(strlen(p) == pos,    "bad char '%c' in '%s'", *(p + pos), p);
}

static void add(char **V) {
  size_t i, j, L;  const char *p;  char S[BUFLEN];  // sum
  CK(NULL != V[0], "vector too short, first element %p", V[0]);
  check_string(V[0]);
  L = strlen(V[0]);
  strncpy(S, V[0], sizeof S);
  for (i = 1; NULL != (p = V[i]); i++) {
    check_string(p);
    CK(strlen(p) == L, "string '%s' bad length:  %lu vs %lu",
                                p,               L, strlen(p));
    for (j = 0; j < L; j++)
      S[j] = A[(let2num(S[j]) + let2num(p[j])) % ALEN];
  }
  printf("%s\n", S);
}

static void subtract(const char *x, const char *y) {
  size_t i, L = strlen(x);  char D[BUFLEN];  // difference
  check_string(x);  check_string(y);
  CK(L == strlen(y), "length mismatch:  %lu vs %lu", L, strlen(y));
  for (i = 0; i < L; i++) {
    int d = let2num(x[i]) - let2num(y[i]);
    if (0 > d) d += ALEN;  // wrap if necessary
    D[i] = A[d];
  }
  D[i] = '\0';
  printf("%s\n", D);
}

// Print random string as long as given string.
static void random_string(const char *p) {
  int c;  size_t i, L = strlen(p);  char B[BUFLEN];
  check_string(p);
  for (i = 0; i < L; i++) {
    // Choose random letter, avoiding modulo bias; stdin provides
    // random bytes, e.g., from /dev/random.  Some input bytes are
    // discarded; stdin must supply adequately many.
    do { c = getchar(); } while (EOF != c && AMOD < c);
    CK(EOF != c, "out of entropy, L == %lu", L);  // maybe "%z"?
    B[i] = A[c % ALEN];
  }
  B[i] = '\0';
  printf("%s\n", B);
}

static void check_digit(const char *p) {
  int i, L = (int)strlen(p), sum = 0;
  check_string(p);
  for (i = 0; i < L; i++)
    sum += let2num(p[i]);
  printf("%c\n", A[sum % ALEN]);
}

int main(int argc, char *argv[]) {
  CK(2 < argc, "argc %d too small", argc);
  switch (argv[1][0]) {
    case '+':  add(2 + argv);                     break;
    case '-':  subtract(argv[2], argv[3]);        break;
    case 'r':  random_string(argv[2]);            break;
    case 'c':  check_digit(argv[2]);              break;
    default:   CK(0, "bad op '%c'", argv[1][0]);  break;
  }
  return 0;
}

