#!/usr/bin/csh -f

# C-Shell script to compile and run am26 program

# For "Drill Bits" column in July/August 2023 issue of
# ACM _Queue_ magazine:  https://queue.acm.org/DrillBits

if (-e warnflags) then
    set wf = `gawk '/^[cx]/{print $2}' warnflags`
else
    echo 'using bare-bones warning flags'
    set wf = '-Wall -Wextra'
endif

if ( -X valgrind ) then
    set v = 'valgrind -q'
else
    echo 'valgrind not available'
    set v = ''
endif

# CFLAGS analogue.  Manually disables a handful of warnings.
set cf = "-std=c11 $wf -Wno-c90-c99-compat"

set b = am26

echo ''
echo 'compile twice: once for max warnings, once for debuggability'
echo '' ; rm -f $b ; gcc $cf -O3 -o $b $b.c ; echo ''
echo '' ; rm -f $b ; gcc $cf -Og -o $b $b.c ; echo ''

echo 'run tests:'
echo ''
echo 'compute check digit on HELLO'
$v ./$b c HELLO
echo ''
echo 'generate random string as long as HELLO'
set   randstr = `$v ./$b r HELLO < /dev/urandom`  # note: urandom != random
echo $randstr
echo ''
echo 'subtract' $randstr 'from HELLO'
set   diff = `$v ./$b - HELLO $randstr`
echo $diff
echo ''
echo 'add' $diff 'back to' $randstr
set   sum = `$v ./$b + $diff $randstr`
echo $sum
if ("HELLO" != $sum) then
    echo 'WARNING:  unexpected sum' $sum ' --- should be HELLO'
endif
echo ''
echo 'sum four-way split from Drill Bits column:'
echo '    TVHTU+DZCFJ+OOFTW+XWXUP=HELLO'
set sum = `$v ./$b + TVHTU DZCFJ OOFTW XWXUP`
echo $sum
if ("HELLO" != $sum) then
    echo 'WARNING:  unexpected sum' $sum ' --- should be HELLO'
endif
echo ''

