// Illustrate modulo bias by collapsing a uniform random number
// to a smaller range the *wrong* way, destroying uniformity.
// See "Drill Bits" column in July/August 2023 issue of ACM
// _Queue_ magazine:  https://queue.acm.org/DrillBits
//
// Author:  Terence Kelly <tpkelly@acm.org>
//
// Moral:  If rand() returns a number uniformly distributed in
// the range [0..RAND_MAX], the expression "rand() % N" where
// N < RAND_MAX is in general *not* uniformly distributed over
// the range [0..N-1].
//
// While we're on the subject of pseudo-random number generators:
// The C language standard permits implementations of the
// standard library to botch rand() horribly.  See my article
// "Catch 23" in ACM _Queue_ magazine, March/April 2023.

#include <stdio.h>
#include <stdlib.h>

int main(int argc, char *argv[]) {
  int MID = RAND_MAX / 3, MAX = 2 * MID, NLo = 0, NHi = 0;
  if (2 == argc)
    srand((unsigned int)atoi(argv[1]));
  for (int i = 0; i < 10000; i++) {
    int r = rand() % MAX;  // due to modulo bias, variable "r"
    if (r < MID) NLo++;    // will *not* be uniformly
    else         NHi++;    // distributed in range [0..MAX-1]
  }
  printf("MID      = %10d\n", MID);
  printf("MAX      = %10d\n", MAX);
  printf("RAND_MAX = %10d\n", RAND_MAX);
  printf("NLo = %d\n", NLo);  // NLo will be roughly twice as
  printf("NHi = %d\n", NHi);  // large as NHi due to modulo bias
  return 0;
}

